<?php

namespace Drupal\ai_healthcheck\Form;

use Drupal\ai_assistant_api\AiAssistantApiRunner;
use Drupal\ai_assistant_api\Data\UserMessage;
use Drupal\ai_assistant_api\Entity\AiAssistant;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\webprofiler\Profiler\Profiler;
use GuzzleHttp\ClientInterface;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webprofiler\DataCollector\HasPanelInterface;
use League\CommonMark\CommonMarkConverter;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\DataCollector\DataCollectorInterface;

class HealthCheck extends FormBase {

  /**
   * {@inheritdoc}
   */
  protected function __construct(
    protected ClientInterface $httpClient,
    protected Profiler $profiler,
    protected AiAssistantApiRunner $aiAssistantClient,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_client'),
      $container->get('webprofiler.profiler'),
      $container->get('ai_assistant_api.runner'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'custom_ai_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $this->profiler->disable();

    // Add the library.
    $form['#attached']['library'][] = 'ai_healthcheck/admin';

    // Set the wrapper ID for the AJAX response.
    $form['#prefix'] = '<div id="ai-healthcheck-form">';
    $form['#suffix'] = '</div>';

    $form['data-container'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['column', 'ai-healthcheck-container']],
    ];

    $form['data-container']['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path'),
      '#description' => $this->t('Enter the path to check.'),
      '#required' => TRUE,
    ];

    $form['data-container']['run'] = [
      '#type' => 'submit',
      '#value' => $this->t('Run Health Check'),
      '#ajax' => [
        'callback' => '::doProfileCallback',
        'wrapper' => 'ai-healthcheck-form',
      ],
    ];

    $form['data-container']['reset'] = [
      '#type' => 'button',
      '#value' => $this->t('Reset'),
      '#attributes' => ['class' => ['visually-hidden']],
    ];

    $form['data-container']['question'] = [
      '#title' => $this->t('Ask a question'),
      '#type' => 'textarea',
      '#rows' => 2,
      '#wrapper_attributes' => ['class' => ['visually-hidden']],
    ];

    $form['data-container']['ask'] = [
      '#type' => 'button',
      '#value' => $this->t('Ask'),
      '#ajax' => [
        'callback' => '::questionCallback',
        'wrapper' => 'ai-text-response',
      ],
      '#attributes' => ['class' => ['visually-hidden']],
    ];

    $form['threadId'] = [
      '#type' => 'hidden',
      '#value' => '',
    ];

    $form['response-container'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['column', 'ai-healthcheck-response-container']],
    ];
    $form['response-container']['response'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'ai-text-response'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function doProfileCallback(array &$form, FormStateInterface $form_state) {
    $assistant = $this->entityTypeManager->getStorage('ai_assistant')->load('healthcheck_assistant');
    if (!$assistant instanceof AiAssistant) {
      return [
        '#type' => 'markup',
        '#markup' => $this->t('No AI assistant found.'),
      ];
    }
    $this->aiAssistantClient->setAssistant($assistant);

    $path = $form_state->getValue('path');

    $message = "Can you profile the path: $path?";

    $conversation = new UserMessage($message);
    $this->aiAssistantClient->setUserMessage($conversation);
    $this->aiAssistantClient->setThrowException(TRUE);

    try {
      $response = $this->aiAssistantClient->process();
      $normalizedResponse = $response->getNormalized();
    }
    catch (\Exception $e) {
      return [
        '#type' => 'markup',
        '#markup' => $this->t('Error: @message', ['@message' => $e->getMessage()]),
      ];
    }

    if (is_object($normalizedResponse) && get_class($normalizedResponse) === ChatMessage::class) {
      $assistantResponseText = $normalizedResponse->getText();
      $this->aiAssistantClient->setAssistantMessage($assistantResponseText);

      // @todo move this somewhere else.
      $form['data-container']['path']['#attributes']['readonly'] = 'readonly';
      $form['data-container']['path']['#attributes']['class'][] = 'disabled';

      $form['data-container']['run']['#attributes']['class'][] = 'visually-hidden';

      $form['data-container']['reset']['#attributes']['class'] = [];
      $form['data-container']['ask']['#attributes']['class'] = [];
      $form['data-container']['question']['#wrapper_attributes']['class'] = [];

      // Output the response.
      $converter = new CommonMarkConverter();
      $form['response-container']['response']['message-1'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $converter->convert($assistantResponseText),
      ];
    }
    return $form;
  }

  public function questionCallback(array &$form, FormStateInterface $form_state) {
    $assistant = $this->entityTypeManager->getStorage('ai_assistant')
      ->load('healthcheck_assistant');
    if (!$assistant instanceof AiAssistant) {
      return [
        '#type' => 'markup',
        '#markup' => $this->t('No AI assistant found.'),
      ];
    }
    $this->aiAssistantClient->setAssistant($assistant);

    $question = $form_state->getValue('question');
    $this->aiAssistantClient->setUserMessage(new UserMessage($question));
    $this->aiAssistantClient->setThrowException(TRUE);

    try {
      $response = $this->aiAssistantClient->process();
      $normalizedResponse = $response->getNormalized();
    }
    catch (\Exception $e) {
      return [
        '#type' => 'markup',
        '#markup' => $this->t('Error: @message', ['@message' => $e->getMessage()]),
      ];
    }

    if (is_object($normalizedResponse) && get_class($normalizedResponse) === ChatMessage::class) {
      $assistantResponseText = $normalizedResponse->getText();
      $this->aiAssistantClient->setAssistantMessage($assistantResponseText);

      // Output the response.
      $converter = new CommonMarkConverter();
      $form['response']['text'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $converter->convert($assistantResponseText),
      ];
    }

    return $form['response'];
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    // @todo Implement submitForm() method.
  }

}
