<?php

namespace Drupal\ai_healthcheck\Plugin\AiFunctionCall;

use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\webprofiler\DataCollector\DatabaseDataCollector;
use Drupal\webprofiler\Profiler\Profiler;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Create a web profile for a given URL.
 */
#[FunctionCall(
  id: 'ai_healthcheck:analyze_collector',
  function_name: 'ai_healthcheck_analyze_collector',
  name: 'Analyze a Webprofiler collector',
  description: 'This method can take a profile ID, load it and provide details for a specific collector.',
  group: 'health_tools',
  context_definitions: [
    'profile_id' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Profile ID"),
      description: new TranslatableMarkup("The ID of the profile to load and get insghts from."),
      required: TRUE
    ),
    'collector' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Collector Name"),
      description: new TranslatableMarkup("The name of the collector to analyze."),
      required: TRUE
    ),
  ]
)]
class AnalyzeCollector extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The web profiler service.
   *
   * @var \Drupal\webprofiler\Profiler\Profiler
   */
  protected Profiler $webProfiler;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer()
    );
    $instance->webProfiler = $container->get('webprofiler.profiler');
    return $instance;
  }

  public function execute() {
    if (!$profileId = $this->getContextValue('profile_id')) {
      throw new \InvalidArgumentException('The profile_id context is required.');
    }

    if (!$collectorName = $this->getContextValue('collector')) {
      throw new \InvalidArgumentException('The collector context is required.');
    }

    if (!$profile = $this->webProfiler->loadProfile($profileId)) {
      throw new \InvalidArgumentException('The profile with the given ID does not exist.');
    }

    if (!$profile->hasCollector($collectorName)) {
      throw new \InvalidArgumentException('The collector with the given name does not exist in the profile.');
    }

    $collector = $profile->getCollector($collectorName);
    $data = $this->extractCollectorData($collector);
    $this->stringOutput = json_encode($data);
  }

  /**
   * Extracts data from a collector.
   *
   * This method uses reflection to get all public methods of the collector,
   * excluding magic methods and methods with required parameters.
   *
   * @param object $collector
   *   The collector object to extract data from.
   *
   * @return array
   *   An associative array of method names and their return values.
   */
  protected function extractCollectorData($collector): array {
    $data = [];

    // @todo either need to do this per collector or have a generic way to handle this.
    // Ideally we could use assistant api and threads with file uploads.
    if ($collector instanceof DatabaseDataCollector) {
      $queries = $collector->getQueries();
      usort($queries, fn($a, $b) => $b['time'] <=> $a['time']);
      $data['topQueries'] = array_slice($queries, 0, 5);
      $data['queryCount'] = $collector->getQueryCount();
      $data['totalQueryTime'] = $collector->getTime();
      return $data;
    }

    $reflection = new \ReflectionClass($collector);
    // Add others if needed.
    $skipMethods = ['reset', 'collect', 'lateCollect'];

    foreach ($reflection->getMethods(\ReflectionMethod::IS_PUBLIC) as $method) {
      $name = $method->getName();

      if (
            str_starts_with($name, '__') ||
            in_array($name, $skipMethods, TRUE) ||
            $method->getDeclaringClass()->getName() !== get_class($collector) ||
            $method->getNumberOfRequiredParameters() > 0
        ) {
        continue;
      }

      try {
        $value = $method->invoke($collector);
        $data[$name] = $value;
      }
      catch (\Throwable $e) {
        $data[$name] = 'Exception: ' . $e->getMessage();
      }
    }

    return $data;
  }

}
