<?php

namespace Drupal\ai_healthcheck\Plugin\AiFunctionCall;

use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use Drupal\webprofiler\DataCollector\HasPanelInterface;
use Drupal\webprofiler\Profiler\Profiler;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\DataCollector\DataCollectorInterface;

/**
 * Create a web profile for a given URL.
 */
#[FunctionCall(
  id: 'ai_healthcheck:analyze_profile',
  function_name: 'ai_healthcheck_analyze_profile',
  name: 'Analyze a Webprofiler profile',
  description: 'This method can take a profile ID, load it and provide insights.',
  group: 'health_tools',
  context_definitions: [
    'profile_id' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Profile ID"),
      description: new TranslatableMarkup("The ID of the profile to load and get insghts from."),
      required: TRUE
    ),
  ]
)]
class AnalyzeProfile extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The web profiler service.
   *
   * @var \Drupal\webprofiler\Profiler\Profiler
   */
  protected Profiler $webProfiler;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer()
    );
    $instance->webProfiler = $container->get('webprofiler.profiler');
    return $instance;
  }

  public function execute() {
    if (!$profileId = $this->getContextValue('profile_id')) {
      throw new \InvalidArgumentException('The profile_id context is required.');
    }

    if (!$profile = $this->webProfiler->loadProfile($profileId)) {
      throw new \InvalidArgumentException('The profile with the given ID does not exist.');
    }

    $collectors = \array_filter($profile->getCollectors(), static fn (DataCollectorInterface $el) => $el instanceof HasPanelInterface);
    if (empty($collectors)) {
      throw new \InvalidArgumentException('No collectors found in the profile.');
    }

    // @todo refactor.
    $message = "Here is a summary of the profile. \n";

    if ($profile->hasCollector('database')) {
      $dbCollector = $profile->getCollector('database');
      $message .= "Database Queries: \n";
      $message .= "Total Queries: " . $dbCollector->getQueryCount() . "\n";
      $message .= "Total Time: " . $dbCollector->getTime() . "\n";
    }

    if ($profile->hasCollector('memory')) {
      $memoryCollector = $profile->getCollector('memory');
      $message .= "Memory Usage: \n";
      $message .= "Peak Memory Usage: " . $memoryCollector->getMemory() . "\n";
    }

    if ($profile->hasCollector('extensions')) {
      $extensionsCollector = $profile->getCollector('extensions');
      $message .= "Modules: " . $extensionsCollector->getModulesCount() . "\n";
      $message .= "Themes: " . $extensionsCollector->getThemesCount() . "\n";
    }

    if ($profile->hasCollector('time')) {
      $timeCollector = $profile->getCollector('time');
      $message .= "Execution Time: \n";
      $message .= "Total Time Duration: " . $timeCollector->getDuration() . "\n";
    }

    if ($profile->hasCollector('views')) {
      $viewsCollector = $profile->getCollector('views');
      $message .= "Views Information: \n";
      $message .= "Total Views: " . $viewsCollector->getViewsCount() . "\n";
    }

    $message .= "And here are the collectors available in the profile:\n";
    foreach ($collectors as $collector) {
      $message .= $collector->getName() . ": \n";
    }

    $this->stringOutput = $message;
  }

}
