<?php

namespace Drupal\ai_healthcheck\Plugin\AiFunctionCall;

use Drupal\Core\Path\PathValidatorInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Create a web profile for a given URL.
 */
#[FunctionCall(
  id: 'ai_healthcheck:create_profile',
  function_name: 'ai_healthcheck_create_profile',
  name: 'Create a Webprofile',
  description: 'This method can take a relative URL and create a webprofile for it.',
  group: 'health_tools',
  context_definitions: [
    'url' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("URL"),
      description: new TranslatableMarkup("The relative url string to create the profile for."),
      required: FALSE
    ),
  ]
)]
class CreateProfile extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * The HTTP client service.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $httpClient;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * The path validator service.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected PathValidatorInterface $pathValidator;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer()
    );
    $instance->httpClient = $container->get('http_client');
    $instance->requestStack = $container->get('request_stack');
    $instance->pathValidator = $container->get('path.validator');
    return $instance;
  }

  public function execute() {
    if (!$url = $this->getContextValue('url')) {
      $url = $this->requestStack->getCurrentRequest()->getRequestUri();
    }

    $path = '/' . ltrim($url, '/');
    // Validate the path.
    if (!$this->pathValidator->isValid($path)) {
      throw new \InvalidArgumentException('The provided path is not valid: ' . $path);
    }

    // Get the full URL of the path.
    $base_url = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();
    $fullUrl = $base_url . $path . '?debug=' . rand(0, 1000);
    try {
      $response = $this->httpClient->request('GET', $fullUrl);
    }
    catch (\Exception $e) {
      throw new \RuntimeException('Failed to fetch the URL: ' . $fullUrl . '. Error: ' . $e->getMessage());
    }

    $headers = $response->getHeaders();

    $headersToCheck = [
      'X-Drupal-Cache-Tags',
      'X-Drupal-Cache-Contexts',
      'X-Debug-Token',
      'X-Debug-Token-Link',
    ];

    $savedHeaders = [];
    foreach ($headersToCheck as $header) {
      if (isset($headers[$header])) {
        $savedHeaders[$header] = $headers[$header];
      }
    }

    $this->stringOutput = "Here are the relevant headers from the response:\n";
    foreach ($savedHeaders as $header => $value) {
      $this->stringOutput .= $header . ": " . implode(", ", $value) . "\n";
    }

  }

}
